"""
Khipu Python Client

A Python client library for the Khipu payment API.
"""

__version__ = "0.1.0"

from khipu.client import Client
from khipu.data_classes import (
    PaymentRequest,
    PaymentResponse,
    ReceiverRequest,
    ReceiverResponse,
)

__all__ = [
    "Client",
    "PaymentRequest",
    "PaymentResponse",
    "ReceiverRequest",
    "ReceiverResponse",
]



from khipu.auth import Auth
from khipu.endpoints import create_payment, create_receiver


class Client:
    def __init__(
        self,
        receiver_id: int,
        secret: str,
        base_url: str = "https://khipu.com/api",
        api_version: str = "2.0",
    ):
        self.base_url = f"{base_url}/{api_version}"
        self.auth = Auth(receiver_id, secret)

    create_payment = create_payment
    create_receiver = create_receiver



"""
Example usage of the Khipu Python client
"""

from khipu import Client, PaymentRequest, ReceiverRequest


def create_simple_payment():
    """Example: Create a simple payment"""

    # Initialize client with your credentials
    client = Client(
        receiver_id=12345,  # Replace with your receiver ID
        secret="your-secret-key",  # Replace with your secret
    )

    # Create a payment request
    payment_request = PaymentRequest(
        amount=1000,  # Amount in smallest currency unit (e.g., cents)
        currency="CLP",
        subject="Test Payment",
    )

    # Create the payment
    response = client.create_payment(payment_request)

    print("Payment created successfully!")
    print(f"Payment ID: {response.payment_id}")
    print(f"Payment URL: {response.payment_url}")
    print(f"\nRedirect your user to: {response.payment_url}")

    return response


def create_detailed_payment():
    """Example: Create a payment with all optional parameters"""

    client = Client(receiver_id=12345, secret="your-secret-key")

    payment_request = PaymentRequest(
        # Required fields
        amount=5000,
        currency="CLP",
        subject="Premium Subscription",
        # Optional fields
        body="Monthly premium subscription for John Doe",
        transaction_id="ORDER-2024-001",
        payer_email="customer@example.com",
        payer_name="John Doe",
        return_url="https://yoursite.com/payment/success",
        cancel_url="https://yoursite.com/payment/cancel",
        notify_url="https://yoursite.com/webhooks/khipu",
        picture_url="https://yoursite.com/images/product.jpg",
        expires_date="2024-12-31T23:59:59",
    )

    response = client.create_payment(payment_request)

    print("Detailed payment created!")
    print(f"Payment URL: {response.payment_url}")

    return response


def create_receiver_account():
    """Example: Create a new receiver account"""

    # Note: Usually you don't need this unless you're creating sub-accounts
    client = Client(receiver_id=12345, secret="your-secret-key")

    receiver_request = ReceiverRequest(
        # Admin information
        admin_first_name="John",
        admin_last_name="Doe",
        admin_email="admin@example.com",
        # Business information
        country_code="CL",
        business_identifier="12345678-9",
        business_category="services",
        business_name="My Business SPA",
        business_phone="+56912345678",
        business_address_line_1="Av. Principal 123",
        business_address_line_2="Oficina 456",
        business_address_line_3="Santiago, Región Metropolitana",
        # Contact information
        contact_full_name="Jane Doe",
        contact_job_title="Finance Manager",
        contact_email="contact@example.com",
        contact_phone="+56987654321",
    )

    response = client.create_receiver(receiver_request)

    print("Receiver created successfully!")
    print(f"Receiver ID: {response.receiver_id}")
    print(f"Secret: {response.secret}")
    print("\nStore these credentials securely!")

    return response


def main():
    """Run examples"""

    print("=" * 60)
    print("Khipu Python Client - Examples")
    print("=" * 60)
    print()

    print("Example 1: Simple Payment")
    print("-" * 60)
    try:
        # Uncomment to run:
        # create_simple_payment()
        print("(Uncomment the function call to run)")
    except Exception as e:
        print(f"Error: {e}")

    print()
    print("Example 2: Detailed Payment")
    print("-" * 60)
    try:
        # Uncomment to run:
        # create_detailed_payment()
        print("(Uncomment the function call to run)")
    except Exception as e:
        print(f"Error: {e}")

    print()
    print("=" * 60)
    print("Remember to replace the credentials with your actual Khipu API keys!")
    print("=" * 60)


if __name__ == "__main__":
    main()



from urllib.parse import quote
from typing import Dict, Any


class Auth:
    def __init__(self, receiver_id: int, secret: str):
        self.receiver_id = receiver_id
        self.secret = secret

    def _percent_encode(self, value: str) -> str:
        if value is None:
            return ""
        return quote(str(value), safe="")

    def _calculate_signature(
        self, method: str, url: str, params: Dict[str, Any]
    ) -> str:
        import hmac
        import hashlib

        to_sign = f"{method.upper()}&{self._percent_encode(url)}"
        sorted_keys = sorted(params.keys())

        for key in sorted_keys:
            to_sign += (
                f"&{self._percent_encode(key)}={self._percent_encode(params[key])}"
            )

        signature = hmac.new(
            self.secret.encode("utf-8"), to_sign.encode("utf-8"), hashlib.sha256
        ).hexdigest()

        return signature

    def _get_auth_header(self, method: str, url: str, params: Dict[str, Any]) -> str:
        signature = self._calculate_signature(method, url, params)
        return f"{self.receiver_id}:{signature}"

    def get_headers(
        self, method: str, url: str, params: Dict[str, Any]
    ) -> Dict[str, str]:
        auth_header = self._get_auth_header(method, url, params)
        return {
            "Authorization": auth_header,
            "Content-Type": "application/x-www-form-urlencoded",
        }



import requests
from dataclasses import asdict
from khipu.data_classes import (
    PaymentResponse,
    ReceiverRequest,
    PaymentRequest,
    ReceiverResponse,
)


def create_payment(self, payment_request: PaymentRequest) -> PaymentResponse:
    url = f"{self.base_url}/payments"
    params = {k: v for k, v in asdict(payment_request).items() if v is not None}
    headers = self.auth.get_headers("POST", url, params)
    response = requests.post(url, data=params, headers=headers)
    response.raise_for_status()
    data = response.json()
    return PaymentResponse(**data)


def create_receiver(self, receiver_request: ReceiverRequest) -> ReceiverResponse:
    url = f"{self.base_url}/receivers"
    params = {k: v for k, v in asdict(receiver_request).items() if v is not None}
    headers = self.auth.get_headers("POST", url, params)
    response = requests.post(url, data=params, headers=headers)
    response.raise_for_status()
    data = response.json()
    return ReceiverResponse(**data)



import os
import requests
from dotenv import load_dotenv
from data_classes import PaymentRequest, ReceiverRequest
from khipu.client import Client

if __name__ == "__main__":
    load_dotenv()
    receiver_id = int(os.getenv("KHIPU_RECEIVER_ID", "0"))
    secret = os.getenv("KHIPU_SECRET", "")
    client = Client(receiver_id=receiver_id, secret=secret)

    # Example 1: Create a payment
    print("=== Creating Payment ===")
    try:
        payment = client.create_payment(
            PaymentRequest(amount=1000, currency="CLP", subject="Cobro de prueba")
        )

        print(payment)

    except requests.exceptions.HTTPError as e:
        print(f"Error creating payment: {e}")
        print(f"Response: {e.response.text}")
    except Exception as e:
        print(f"Unexpected error: {e}")

    # Example 2: Create a receiver
    print("=== Creating Receiver ===")
    try:
        receiver = client.create_receiver(
            ReceiverRequest(
                admin_first_name="Nombre",
                admin_last_name="Apellido",
                admin_email="admin@email.com",
                country_code="CL",
                business_identifier="99.999.999-9",
                business_category="VENTA AL POR MAYOR DE FRUTAS Y VERDURAS",
                business_name="Nombre Tributario",
                business_phone="+56988887777",
                business_address_line_1="Calle principal 1111",
                business_address_line_2="Oficina 3-A",
                business_address_line_3="Santiago",
                contact_full_name="Nombre Contacto",
                contact_job_title="Tesorero",
                contact_email="contacto@email.com",
                contact_phone="+56955553333",
                notify_url="http://micomercio.com/account/notify_url",
                rendition_url="http://micomercio.com/account/rendition_url",
            )
        )

        print(receiver)

    except requests.exceptions.HTTPError as e:
        print(f"Error creating receiver: {e}")
        print(f"Response: {e.response.text}")
    except Exception as e:
        print(f"Unexpected error: {e}")



"""
Khipu Client - Main entry point for CLI usage (optional)
"""

from khipu.client import Client


def main(receiver_id: int, secret: str) -> Client:
    """
    Create and return a Khipu client instance.

    Args:
        receiver_id: Your Khipu receiver ID
        secret: Your Khipu secret key

    Returns:
        Client: Configured Khipu client instance
    """
    return Client(receiver_id=receiver_id, secret=secret)



from typing import Optional
from dataclasses import dataclass


@dataclass
class BaseDataClass:
    pass


@dataclass
class PaymentResponse(BaseDataClass):
    payment_id: str
    payment_url: str
    simplified_transfer_url: str
    transfer_url: str
    app_url: str
    ready_for_terminal: bool


@dataclass
class PaymentRequest(BaseDataClass):
    amount: int
    currency: str
    subject: str
    transaction_id: Optional[str] = None
    custom: Optional[str] = None
    body: Optional[str] = None
    bank_id: Optional[str] = None
    return_url: Optional[str] = None
    cancel_url: Optional[str] = None
    picture_url: Optional[str] = None
    notify_url: Optional[str] = None
    contract_url: Optional[str] = None
    notify_api_version: Optional[str] = None
    expires_date: Optional[str] = None
    send_email: Optional[bool] = None
    payer_name: Optional[str] = None
    payer_email: Optional[str] = None
    send_reminders: Optional[bool] = None
    responsible_user_email: Optional[str] = None
    fixed_payer_personal_identifier: Optional[str] = None
    integrator_fee: Optional[int] = None
    collect_account_uuid: Optional[str] = None
    confirm_timeout_date: Optional[str] = None
    mandatory_payment_method: Optional[str] = None


@dataclass
class ReceiverRequest(BaseDataClass):
    admin_first_name: str
    admin_last_name: str
    admin_email: str
    country_code: str
    business_identifier: str
    business_category: str
    business_name: str
    business_phone: str
    business_address_line_1: str
    business_address_line_2: str
    business_address_line_3: str
    contact_full_name: str
    contact_job_title: str
    contact_email: str
    contact_phone: str
    bank_account_bank_id: Optional[str] = None
    bank_account_type: Optional[str] = None
    bank_account_identifier: Optional[str] = None
    bank_account_name: Optional[str] = None
    bank_account_number: Optional[str] = None
    notify_url: Optional[str] = None
    rendition_url: Optional[str] = None


@dataclass
class ReceiverResponse(BaseDataClass):
    receiver_id: str
    secret: str
    contract_url: Optional[str] = None
